const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const deepClone = require('lodash/cloneDeep');
const getBaseConfig = require('./webpack.base');
const getBabelConfig = require('./getBabelConfig');
const formatWinPath = require('../utils/formatWinPath');
const TYPESCRIPT_LOADER = require.resolve('ts-loader');
const BABEL_LOADER = require.resolve('babel-loader');

module.exports = (conf = {}) => {
  const babelConfig = getBabelConfig(conf);
  const baseConfig = getBaseConfig('production', conf);

  baseConfig.devtool(false);

  // 生产环境打包文件名以hash命名
  const fileName = baseConfig.output.get('filename');
  let pathArray = fileName.split('/');
  pathArray.pop(); // pop filename
  pathArray = pathArray.filter((v) => v);
  const outputPath = pathArray.length ? pathArray.join('/') : '';
  baseConfig
    .output
    .filename(formatWinPath(path.join(outputPath, `[name].['hash:6'].js`)));

  baseConfig
    .plugin('MiniCssExtractPlugin')
    .tap((args) => [Object.assign(...args, {
      filename: formatWinPath(path.join(outputPath, `[name].['hash:6'].css`)),
    })]);

  // jsx loader 打包生产环境去掉exclude node_modules 因为使用第三方插件，压缩时可能报错
  baseConfig.module.rule('jsx')
    .test(/\.(jsx|js)?$/)
    .use('babel-loader')
    .loader(BABEL_LOADER)
    .options(Object.assign({}, deepClone(babelConfig), { cacheDirectory: true }));

  // tsx loader
  baseConfig.module.rule('tsx')
    .test(/\.(tsx|ts)?$/)
    .use('babel-loader')
    .loader(BABEL_LOADER)
    .options(Object.assign({}, deepClone(babelConfig), { cacheDirectory: true }))
    .end()
    .use('ts-loader')
    .loader(TYPESCRIPT_LOADER)
    .options({ transpileOnly: true });

  // 压缩文件
  baseConfig.optimization
    .minimizer('UglifyJsPlugin')
    .use(UglifyJsPlugin, [{
      sourceMap: false,
      cache: true,
      parallel: true,
      uglifyOptions: {
        compress: {
          unused: false
        },
        output: {
          ascii_only: true,
          comments: 'some',
          beautify: false,
        },
        mangle: true,
      },
    }]);

  // optimize css file
  baseConfig.optimization
    .minimizer('OptimizeCSSAssetsPlugin')
    .use(OptimizeCSSAssetsPlugin, [{
      cssProcessorOptions: {
        cssDeclarationSorter: false,
        reduceIdents: false,
        parser: require('postcss-safe-parser'),
      },
    }]);

  return baseConfig;
};
